@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-#   Copyright (c) 2008-2009, David P. D. Moss. All rights reserved.
+#   Copyright (c) 2008-2015, David P. D. Moss. All rights reserved.
 #
 #   Released under the BSD license. See the LICENSE file for details.
 #-----------------------------------------------------------------------------
@@ -8,58 +8,92 @@ Classes and functions for dealing with MAC addresses, EUI-48, EUI-64, OUI, IAB
 identifiers.
 """
 
-import sys as _sys
-import os as _os
-import os.path as _path
-import re as _re
-import csv as _csv
-
-import pprint as _pprint
-
 from netaddr.core import NotRegisteredError, AddrFormatError, \
     AddrConversionError, Subscriber, Publisher, DictDotLookup
 from netaddr.strategy import eui48 as _eui48, eui64 as _eui64
 from netaddr.strategy.eui48 import mac_eui48
 from netaddr.ip import IPAddress
 
-#-----------------------------------------------------------------------------
-class OUI(object):
+from netaddr.compat import _is_int, _is_str
+
+
+class BaseIdentifier(object):
+    """Base class for all IEEE identifiers."""
+    __slots__ = ('_value',)
+
+    def __init__(self):
+        self._value = None
+
+    def __int__(self):
+        """:return: integer value of this identifier"""
+        return self._value
+
+    def __long__(self):
+        """:return: integer value of this identifier"""
+        return self._value
+
+    def __oct__(self):
+        """:return: octal string representation of this identifier."""
+        #   Python 2.x only.
+        if self._value == 0:
+            return '0'
+        return '0%o' % self._value
+
+    def __hex__(self):
+        """:return: hexadecimal string representation of this identifier."""
+        #   Python 2.x only.
+        return '0x%x' % self._value
+
+    def __index__(self):
+        """
+        :return: return the integer value of this identifier when passed to
+            hex(), oct() or bin().
+        """
+        #   Python 3.x only.
+        return self._value
+
+
+class OUI(BaseIdentifier):
     """
     An individual IEEE OUI (Organisationally Unique Identifier).
 
-    For online details see - U{http://standards.ieee.org/regauth/oui/}
+    For online details see - http://standards.ieee.org/regauth/oui/
+
     """
+    __slots__ = ('records',)
+
     def __init__(self, oui):
         """
         Constructor
 
-        @param oui: an OUI string C{XX-XX-XX} or an unsigned integer.
-            Also accepts and parses full MAC/EUI-48 address strings (but not
+        :param oui: an OUI string ``XX-XX-XX`` or an unsigned integer. \
+            Also accepts and parses full MAC/EUI-48 address strings (but not \
             MAC/EUI-48 integers)!
         """
+        super(OUI, self).__init__()
+
         #   Lazy loading of IEEE data structures.
         from netaddr.eui import ieee
 
-        self.value = None
         self.records = []
 
         if isinstance(oui, str):
             #TODO: Improve string parsing here.
             #TODO: Accept full MAC/EUI-48 addressses as well as XX-XX-XX
             #TODO: and just take /16 (see IAB for details)
-            self.value = int(oui.replace('-', ''), 16)
-        elif isinstance(oui, (int, long)):
+            self._value = int(oui.replace('-', ''), 16)
+        elif _is_int(oui):
             if 0 <= oui <= 0xffffff:
-                self.value = oui
+                self._value = oui
             else:
                 raise ValueError('OUI int outside expected range: %r' % oui)
         else:
             raise TypeError('unexpected OUI format: %r' % oui)
 
         #   Discover offsets.
-        if self.value in ieee.OUI_INDEX:
-            fh = open(ieee.OUI_REGISTRY, 'rb')
-            for (offset, size) in ieee.OUI_INDEX[self.value]:
+        if self._value in ieee.OUI_INDEX:
+            fh = open(ieee.OUI_REGISTRY)
+            for (offset, size) in ieee.OUI_INDEX[self._value]:
                 fh.seek(offset)
                 data = fh.read(size)
                 self._parse_data(data, offset, size)
@@ -67,6 +101,30 @@ class OUI(object):
         else:
             raise NotRegisteredError('OUI %r not registered!' % oui)
 
+    def __eq__(self, other):
+        if not isinstance(other, OUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return self._value == other._value
+
+    def __ne__(self, other):
+        if not isinstance(other, OUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return self._value != other._value
+
+    def __getstate__(self):
+        """:returns: Pickled state of an `OUI` object."""
+        return self._value, self.records
+
+    def __setstate__(self, state):
+        """:param state: data used to unpickle a pickled `OUI` object."""
+        self._value, self.records = state
+
     def _parse_data(self, data, offset, size):
         """Returns a dict record from raw OUI record data"""
         record = {
@@ -80,12 +138,12 @@ class OUI(object):
 
         for line in data.split("\n"):
             line = line.strip()
-            if line == '':
+            if not line:
                 continue
 
             if '(hex)' in line:
-                record['idx'] = self.value
-                record['org'] = ' '.join(line.split()[2:])
+                record['idx'] = self._value
+                record['org'] = line.split(None, 2)[2]
                 record['oui'] = str(self)
             elif '(base 16)' in line:
                 continue
@@ -99,56 +157,46 @@ class OUI(object):
         """Number of registered organisations with this OUI"""
         return len(self.records)
 
-    def __int__(self):
-        """@return: integer representation of this OUI"""
-        return self.value
-
-    def __hex__(self):
-        """
-        @return: hexadecimal string representation of this OUI (in network byte
-        order).
-        """
-        return hex(self.value).rstrip('L').lower()
-
     def registration(self, index=0):
         """
         The IEEE registration details for this OUI.
 
-        @param index: the index of record (may contain multiple registrations)
+        :param index: the index of record (may contain multiple registrations)
             (Default: 0 - first registration)
 
-        @return: Objectified Python data structure containing registration
+        :return: Objectified Python data structure containing registration
             details.
         """
         return DictDotLookup(self.records[index])
 
     def __str__(self):
-        """@return: string representation of this OUI"""
-        int_val = self.value
-        words = []
-        for _ in range(3):
-            word = int_val & 0xff
-            words.append('%02x' % word)
-            int_val >>= 8
-        return '-'.join(reversed(words)).upper()
+        """:return: string representation of this OUI"""
+        int_val = self._value
+        return "%02X-%02X-%02X" % (
+                (int_val >> 16) & 0xff,
+                (int_val >> 8) & 0xff,
+                int_val & 0xff)
 
     def __repr__(self):
-        """@return: executable Python string to recreate equivalent object."""
-        return "%s('%s')" % (self.__class__.__name__, self)
+        """:return: executable Python string to recreate equivalent object."""
+        return "OUI('%s')" % self
 
-#-----------------------------------------------------------------------------
-class IAB(object):
+
+class IAB(BaseIdentifier):
     """
     An individual IEEE IAB (Individual Address Block) identifier.
 
-    For online details see - U{http://standards.ieee.org/regauth/oui/}
+    For online details see - http://standards.ieee.org/regauth/oui/
+
     """
+    __slots__ = ('record',)
+
     @staticmethod
     def split_iab_mac(eui_int, strict=False):
         """
-        @param eui_int: a MAC IAB as an unsigned integer.
+        :param eui_int: a MAC IAB as an unsigned integer.
 
-        @param strict: If True, raises a ValueError if the last 12 bits of
+        :param strict: If True, raises a ValueError if the last 12 bits of
             IAB MAC/EUI-48 address are non-zero, ignores them otherwise.
             (Default: False)
         """
@@ -172,18 +220,19 @@ class IAB(object):
         """
         Constructor
 
-        @param iab: an IAB string C{00-50-C2-XX-X0-00} or an unsigned integer.
-            This address looks like an EUI-48 but it should not have any
-            non-zero bits in the last 3 bytes.
+        :param iab: an IAB string ``00-50-C2-XX-X0-00`` or an unsigned \
+            integer. This address looks like an EUI-48 but it should not \
+            have any non-zero bits in the last 3 bytes.
 
-        @param strict: If True, raises a ValueError if the last 12 bits of
-            IAB MAC/EUI-48 address are non-zero, ignores them otherwise.
+        :param strict: If True, raises a ValueError if the last 12 bits \
+            of IAB MAC/EUI-48 address are non-zero, ignores them otherwise. \
             (Default: False)
         """
+        super(IAB, self).__init__()
+
         #   Lazy loading of IEEE data structures.
         from netaddr.eui import ieee
 
-        self.value = None
         self.record = {
             'idx': 0,
             'iab': '',
@@ -198,18 +247,19 @@ class IAB(object):
             #TODO: '00-50-C2' is actually invalid.
             #TODO: Should be '00-50-C2-00-00-00' (i.e. a full MAC/EUI-48)
             int_val = int(iab.replace('-', ''), 16)
-            (iab_int, user_int) = IAB.split_iab_mac(int_val, strict)
-            self.value = iab_int
-        elif isinstance(iab, (int, long)):
-            (iab_int, user_int) = IAB.split_iab_mac(iab, strict)
-            self.value = iab_int
+            iab_int, user_int = self.split_iab_mac(int_val,
+                    strict=strict)
+            self._value = iab_int
+        elif _is_int(iab):
+            iab_int, user_int = self.split_iab_mac(iab, strict=strict)
+            self._value = iab_int
         else:
             raise TypeError('unexpected IAB format: %r!' % iab)
 
         #   Discover offsets.
-        if self.value in ieee.IAB_INDEX:
-            fh = open(ieee.IAB_REGISTRY, 'rb')
-            (offset, size) = ieee.IAB_INDEX[self.value][0]
+        if self._value in ieee.IAB_INDEX:
+            fh = open(ieee.IAB_REGISTRY)
+            (offset, size) = ieee.IAB_INDEX[self._value][0]
             self.record['offset'] = offset
             self.record['size'] = size
             fh.seek(offset)
@@ -219,53 +269,67 @@ class IAB(object):
         else:
             raise NotRegisteredError('IAB %r not unregistered!' % iab)
 
+    def __eq__(self, other):
+        if not isinstance(other, IAB):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return self._value == other._value
+
+    def __ne__(self, other):
+        if not isinstance(other, IAB):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return self._value != other._value
+
+    def __getstate__(self):
+        """:returns: Pickled state of an `IAB` object."""
+        return self._value, self.record
+
+    def __setstate__(self, state):
+        """:param state: data used to unpickle a pickled `IAB` object."""
+        self._value, self.record = state
+
     def _parse_data(self, data, offset, size):
         """Returns a dict record from raw IAB record data"""
         for line in data.split("\n"):
             line = line.strip()
-            if line == '':
+            if not line:
                 continue
 
             if '(hex)' in line:
-                self.record['idx'] = self.value
-                self.record['org'] = ' '.join(line.split()[2:])
+                self.record['idx'] = self._value
+                self.record['org'] = line.split(None, 2)[2]
                 self.record['iab'] = str(self)
             elif '(base 16)' in line:
                 continue
             else:
                 self.record['address'].append(line)
 
-    def __int__(self):
-        """@return: integer representation of this IAB"""
-        return self.value
-
-    def __hex__(self):
-        """
-        @return: hexadecimal string representation of this IAB (in network
-            byte order)
-        """
-        return hex(self.value).rstrip('L').lower()
-
     def registration(self):
-        """ The IEEE registration details for this IAB"""
+        """The IEEE registration details for this IAB"""
         return DictDotLookup(self.record)
 
     def __str__(self):
-        """@return: string representation of this IAB"""
-        int_val = self.value << 12
-        words = []
-        for _ in range(6):
-            word = int_val & 0xff
-            words.append('%02x' % word)
-            int_val >>= 8
-        return '-'.join(reversed(words)).upper()
+        """:return: string representation of this IAB"""
+        int_val = self._value << 4
+
+        return "%02X-%02X-%02X-%02X-%02X-00" % (
+                (int_val >> 32) & 0xff,
+                (int_val >> 24) & 0xff,
+                (int_val >> 16) & 0xff,
+                (int_val >> 8) & 0xff,
+                int_val & 0xff)
 
     def __repr__(self):
-        """@return: executable Python string to recreate equivalent object."""
-        return "%s('%s')" % (self.__class__.__name__, self)
+        """:return: executable Python string to recreate equivalent object."""
+        return "IAB('%s')" % self
 
-#-----------------------------------------------------------------------------
-class EUI(object):
+
+class EUI(BaseIdentifier):
     """
     An IEEE EUI (Extended Unique Identifier).
 
@@ -273,22 +337,27 @@ class EUI(object):
 
     Input parsing for EUI-48 addresses is flexible, supporting many MAC
     variants.
+
     """
+    __slots__ = ('_module', '_dialect')
+
     def __init__(self, addr, version=None, dialect=None):
         """
         Constructor.
 
-        @param addr: an EUI-48 (MAC) or EUI-64 address in string format or an
-        unsigned integer. May also be another EUI object (copy construction).
+        :param addr: an EUI-48 (MAC) or EUI-64 address in string format or \
+            an unsigned integer. May also be another EUI object (copy \
+            construction).
 
-        @param version: (optional) the explict EUI address version. Mainly
-            used to distinguish between EUI-48 and EUI-64 identifiers
+        :param version: (optional) the explicit EUI address version, either \
+            48 or 64. Mainly used to distinguish EUI-48 and EUI-64 identifiers \
             specified as integers which may be numerically equivalent.
 
-        @param dialect: (optional) the mac_* dialect to be used to configure
+        :param dialect: (optional) the mac_* dialect to be used to configure \
             the formatting of EUI-48 (MAC) addresses.
         """
-        self._value = None
+        super(EUI, self).__init__()
+
         self._module = None
 
         if isinstance(addr, EUI):
@@ -311,16 +380,40 @@ class EUI(object):
         else:
         #   Choose a default version when addr is an integer and version is
         #   not specified.
-            if 0 <= addr <= 0xffffffffffff:
-                self._module = _eui48
-            elif 0xffffffffffff < addr <= 0xffffffffffffffff:
-                self._module = _eui64
+            if _is_int(addr):
+                if 0 <= addr <= 0xffffffffffff:
+                    self._module = _eui48
+                elif 0xffffffffffff < addr <= 0xffffffffffffffff:
+                    self._module = _eui64
 
         self.value = addr
 
         #   Choose a dialect for MAC formatting.
         self.dialect = dialect
 
+    def __getstate__(self):
+        """:returns: Pickled state of an `EUI` object."""
+        return self._value, self._module.version, self.dialect
+
+    def __setstate__(self, state):
+        """
+        :param state: data used to unpickle a pickled `EUI` object.
+
+        """
+        value, version, dialect = state
+
+        self._value = value
+
+        if version == 48:
+            self._module = _eui48
+        elif version == 64:
+            self._module = _eui64
+        else:
+            raise ValueError('unpickling failed for object state: %s' \
+                % str(state))
+
+        self.dialect = dialect
+
     def _get_value(self):
         return self._value
 
@@ -346,7 +439,7 @@ class EUI(object):
                     % value)
         else:
             #   EUI version is explicit.
-            if hasattr(value, 'upper'):
+            if _is_str(value):
                 try:
                     self._value = self._module.str_to_int(value)
                 except AddrFormatError:
@@ -389,19 +482,19 @@ class EUI(object):
     def ei(self):
         """The EI (Extension Identifier) for this EUI"""
         if self._module == _eui48:
-            return '-'.join(["%02x" % i for i in self[3:6]]).upper()
+            return '%02X-%02X-%02X' % tuple(self[3:6])
         elif self._module == _eui64:
-            return '-'.join(["%02x" % i for i in self[3:8]]).upper()
+            return '%02X-%02X-%02X-%02X-%02X' % tuple(self[3:8])
 
     def is_iab(self):
-        """@return: True if this EUI is an IAB address, False otherwise"""
+        """:return: True if this EUI is an IAB address, False otherwise"""
         return 0x50c2000 <= (self._value >> 12) <= 0x50c2fff
 
     @property
     def iab(self):
         """
         If is_iab() is True, the IAB (Individual Address Block) is returned,
-        C{None} otherwise.
+        ``None`` otherwise.
         """
         if self.is_iab():
             return IAB(self._value >> 12)
@@ -413,12 +506,12 @@ class EUI(object):
 
     def __getitem__(self, idx):
         """
-        @return: The integer value of the word referenced by index (both
-            positive and negative). Raises C{IndexError} if index is out
-            of bounds. Also supports Python list slices for accessing
+        :return: The integer value of the word referenced by index (both \
+            positive and negative). Raises ``IndexError`` if index is out \
+            of bounds. Also supports Python list slices for accessing \
             word groups.
         """
-        if isinstance(idx, (int, long)):
+        if _is_int(idx):
             #   Indexing, including negative indexing goodness.
             num_words = self._dialect.num_words
             if not (-num_words) <= idx <= (num_words - 1):
@@ -431,18 +524,18 @@ class EUI(object):
             raise TypeError('unsupported type %r!' % idx)
 
     def __setitem__(self, idx, value):
-        """Sets the value of the word referenced by index in this address"""
+        """Set the value of the word referenced by index in this address"""
         if isinstance(idx, slice):
             #   TODO - settable slices.
             raise NotImplementedError('settable slices are not supported!')
 
-        if not isinstance(idx, (int, long)):
+        if not _is_int(idx):
             raise TypeError('index not an integer!')
 
         if not 0 <= idx <= (self._dialect.num_words - 1):
             raise IndexError('index %d outside address type boundary!' % idx)
 
-        if not isinstance(value, (int, long)):
+        if not _is_int(value):
             raise TypeError('value not an integer!')
 
         if not 0 <= value <= self._dialect.max_word:
@@ -454,144 +547,166 @@ class EUI(object):
         self._value = self._module.words_to_int(words)
 
     def __hash__(self):
-        """@return: hash of this EUI object suitable for dict keys, sets etc"""
+        """:return: hash of this EUI object suitable for dict keys, sets etc"""
         return hash((self.version, self._value))
 
-    def __int__(self):
-        """@return: value of this EUI object as an unsigned integer"""
-        return self._value
-
-    def __long__(self):
-        """@return: value of this EUI object as an unsigned integer"""
-        return self._value
-
-    def __hex__(self):
-        """
-        @return: hexadecimal string representation of this EUI identifier.
-        """
-        return hex(self._value).rstrip('L').lower()
-
     def __eq__(self, other):
         """
-        @return: C{True} if this EUI object is numerically the same as other,
-            C{False} otherwise.
+        :return: ``True`` if this EUI object is numerically the same as other, \
+            ``False`` otherwise.
         """
-        try:
-            return(self.version, self._value) == (other.version, other._value)
-        except AttributeError:
-            return NotImplemented
+        if not isinstance(other, EUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return (self.version, self._value) == (other.version, other._value)
 
     def __ne__(self, other):
         """
-        @return: C{False} if this EUI object is numerically the same as the
-            other, C{True} otherwise.
+        :return: ``True`` if this EUI object is numerically the same as other, \
+            ``False`` otherwise.
         """
-        try:
-            return(self.version, self._value) != (other.version, other._value)
-        except AttributeError:
-            return NotImplemented
+        if not isinstance(other, EUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return (self.version, self._value) != (other.version, other._value)
 
     def __lt__(self, other):
         """
-        @return: C{True} if this EUI object is numerically lower in value than
-            other, C{False} otherwise.
+        :return: ``True`` if this EUI object is numerically lower in value than \
+            other, ``False`` otherwise.
         """
-        try:
-            return (self.version, self._value) < (other.version, other._value)
-        except AttributeError:
-            return NotImplemented
+        if not isinstance(other, EUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return (self.version, self._value) < (other.version, other._value)
 
     def __le__(self, other):
         """
-        @return: C{True} if this EUI object is numerically lower or equal in
-            value to other, C{False} otherwise.
+        :return: ``True`` if this EUI object is numerically lower or equal in \
+            value to other, ``False`` otherwise.
         """
-        try:
-            return(self.version, self._value) <= (other.version, other._value)
-        except AttributeError:
-            return NotImplemented
+        if not isinstance(other, EUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return (self.version, self._value) <= (other.version, other._value)
 
     def __gt__(self, other):
         """
-        @return: C{True} if this EUI object is numerically greater in value than
-            other, C{False} otherwise.
+        :return: ``True`` if this EUI object is numerically greater in value \
+            than other, ``False`` otherwise.
         """
-        try:
-            return (self.version, self._value) > (other.version, other._value)
-        except AttributeError:
-            return NotImplemented
+        if not isinstance(other, EUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return (self.version, self._value) > (other.version, other._value)
 
     def __ge__(self, other):
         """
-        @return: C{True} if this EUI object is numerically greater or equal in
-            value to other, C{False} otherwise.
+        :return: ``True`` if this EUI object is numerically greater or equal \
+            in value to other, ``False`` otherwise.
         """
-        try:
-            return(self.version, self._value) >= (other.version, other._value)
-        except AttributeError:
-            return NotImplemented
+        if not isinstance(other, EUI):
+            try:
+                other = self.__class__(other)
+            except Exception:
+                return NotImplemented
+        return (self.version, self._value) >= (other.version, other._value)
 
     def bits(self, word_sep=None):
         """
-        @param word_sep: (optional) the separator to insert between words.
+        :param word_sep: (optional) the separator to insert between words. \
             Default: None - use default separator for address type.
 
-        @return: human-readable binary digit string of this address"""
+        :return: human-readable binary digit string of this address.
+        """
         return self._module.int_to_bits(self._value, word_sep)
 
     @property
     def packed(self):
-        """@return: binary packed string of this address"""
+        """The value of this EUI address as a packed binary string."""
         return self._module.int_to_packed(self._value)
 
     @property
     def words(self):
-        """
-        A list of unsigned integer octets found in this EUI address.
-        """
+        """A list of unsigned integer octets found in this EUI address."""
         return self._module.int_to_words(self._value)
 
     @property
     def bin(self):
         """
-        @return: standard Python binary representation of this address. A back
-            port of the format provided by the builtin bin() type available in
-            Python 2.6.x and higher."""
+        The value of this EUI adddress in standard Python binary
+        representational form (0bxxx). A back port of the format provided by
+        the builtin bin() function found in Python 2.6.x and higher.
+        """
         return self._module.int_to_bin(self._value)
 
     def eui64(self):
         """
-        @return: The value of this EUI object as a new 64-bit EUI object.
-            - If this object represents an EUI-48 it is converted to EUI-64
-                as per the standard.
-            - If this object is already and EUI-64, it just returns a new,
-                numerically equivalent object is returned instead.
+        - If this object represents an EUI-48 it is converted to EUI-64 \
+            as per the standard.
+        - If this object is already an EUI-64, a new, numerically \
+            equivalent object is returned instead.
+
+        :return: The value of this EUI object as a new 64-bit EUI object.
         """
         if self.version == 48:
-            eui64_words = ["%02x" % i for i in self[0:3]] + ['ff', 'fe'] + \
-                     ["%02x" % i for i in self[3:6]]
-
-            return self.__class__('-'.join(eui64_words))
+            # Convert 11:22:33:44:55:66 into 11:22:33:FF:FE:44:55:66.
+            first_three = self._value >> 24
+            last_three = self._value & 0xffffff
+            new_value = (first_three << 40) | 0xfffe000000 | last_three
         else:
-            return EUI(str(self))
+            # is already a EUI64
+            new_value = self._value
+        return self.__class__(new_value, version=64)
+
+    def modified_eui64(self):
+        """
+        - create a new EUI object with a modified EUI-64 as described in RFC 4291 section 2.5.1
+
+        :return: a new and modified 64-bit EUI object.
+        """
+        # Modified EUI-64 format interface identifiers are formed by inverting
+        # the "u" bit (universal/local bit in IEEE EUI-64 terminology) when
+        # forming the interface identifier from IEEE EUI-64 identifiers.  In
+        # the resulting Modified EUI-64 format, the "u" bit is set to one (1)
+        # to indicate universal scope, and it is set to zero (0) to indicate
+        # local scope.
+        eui64 = self.eui64()
+        eui64._value ^= 0x00000000000000000200000000000000
+        return eui64
+
+    def ipv6(self, prefix):
+        """
+        .. note:: This poses security risks in certain scenarios. \
+            Please read RFC 4941 for details. Reference: RFCs 4291 and 4941.
+
+        :param prefix: ipv6 prefix
+
+        :return: new IPv6 `IPAddress` object based on this `EUI` \
+            using the technique described in RFC 4291.
+        """
+        int_val = int(prefix) + int(self.modified_eui64())
+        return IPAddress(int_val, version=6)
 
     def ipv6_link_local(self):
         """
-        @return: new link local IPv6 L{IPAddress} object based on this L{EUI}
-            using the technique described in RFC 4291. B{Please Note:} this
-            poses security risks in certain scenarios. Please read RFC 4941 for
-            details. Reference: RFCs 4291 and 4941.
+        .. note:: This poses security risks in certain scenarios. \
+            Please read RFC 4941 for details. Reference: RFCs 4291 and 4941.
+
+        :return: new link local IPv6 `IPAddress` object based on this `EUI` \
+            using the technique described in RFC 4291.
         """
-        int_val = 0xfe800000000000000200000000000000
-
-        if self.version == 48:
-            eui64_tokens = ["%02x" % i for i in self[0:3]] + ['ff', 'fe'] + \
-                ["%02x" % i for i in self[3:6]]
-            int_val += int(''.join(eui64_tokens), 16)
-        else:
-            int_val += self._value
-
-        return IPAddress(int_val, 6)
+        return self.ipv6(0xfe800000000000000000000000000000)
 
     @property
     def info(self):
@@ -606,9 +721,10 @@ class EUI(object):
         return DictDotLookup(data)
 
     def __str__(self):
-        """@return: EUI in representational format"""
+        """:return: EUI in representational format"""
         return self._module.int_to_str(self._value, self._dialect)
 
     def __repr__(self):
-        """@return: Python statement to create equivalent object"""
-        return "%s('%s')" % (self.__class__.__name__, self)
+        """:return: executable Python string to recreate equivalent object."""
+        return "EUI('%s')" % self
+
